"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.treeModelApplyAction = exports.removeElementToArr = void 0;

var _actionType = require("./actionType");

var _this = void 0;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var newNodeTemplate = {
  data: {},
  depth: null,
  height: 0,
  parent: null,
  x: 0,
  y: 0
};
var index_newObj = 1;

var removeElementToArr = function removeElementToArr(arrayList, elementName) {
  if (!arrayList) return undefined;
  var index = arrayList.findIndex(function (item) {
    return item === elementName || item.name === elementName;
  });

  if (index > -1) {
    arrayList.splice(index, 1);
    return elementName;
  }

  return undefined;
};

exports.removeElementToArr = removeElementToArr;

function initializeNewNode(type) {
  var newObj = Object.assign({}, newNodeTemplate);
  var newName = 'new_' + index_newObj++;

  var indexNewObj = _defineProperty({
    name: newName,
    "label": "NEW NODE",
    type: 'node'
  }, "type", type);

  newObj.data = indexNewObj;
  return newObj;
}

var treeModelApplyAction = function treeModelApplyAction(nodeName, actionName, graphData, nodeIndex) {
  if (!nodeName) return false; //const newName='new_'+index_newObj++;

  var currentNode = _this.state.graphData[nodeName];
  var currentDataProviderNode = _this.nodeIndex[nodeName];

  var indexNewObj = _defineProperty({
    name: newName,
    "label": "NEW NODE",
    type: 'node'
  }, "type", currentNode.data.type);

  if (actionName === _actionType.NODE_ACTION_NAME.ADD_NEW_ENTITY) {
    indexNewObj.type = 'EntityClass';
    actionName = _actionType.NODE_ACTION_NAME.ADD_CHILD;
  } else if (actionName === _actionType.NODE_ACTION_NAME.ADD_NEW_CLASS) {
    indexNewObj.type = 'OrdinaryClass';
    actionName = _actionType.NODE_ACTION_NAME.ADD_CHILD;
  } else if (actionName === _actionType.NODE_ACTION_NAME.ADD_NEW_RELATIONSHIP) {
    indexNewObj.type = 'Relationship';
    actionName = _actionType.NODE_ACTION_NAME.ADD_CHILD;
  } //this.nodeIndex[newName]=indexNewObj;


  switch (actionName) {
    case _actionType.NODE_ACTION_NAME.ADD_CHILD:
      newObj.parent = currentNode;
      newObj.data = indexNewObj; //{"name": newName, "label": "NEW NODE"}

      newObj.depth = currentNode.depth + 1;
      newObj.x = currentNode.x - 100;
      newObj.y = currentNode.y + 100;
      if (!currentNode.children) currentNode.children = [];
      currentNode.children.push(newObj);
      _this.state.graphData[newName] = newObj;
      if (!currentDataProviderNode.children) currentDataProviderNode.children = [];
      currentDataProviderNode.children.push(indexNewObj);
      _this.nodeIndex[newName] = indexNewObj;
      _this.state.dataEdges[currentNode.data.name + '_' + newObj.data.name] = {
        source: currentNode.data.name,
        target: newObj.data.name
      };

      _this.state.nodes.push(newObj);

      break;

    case _actionType.NODE_ACTION_NAME.ADD_PARENT:
      var parentObj = currentNode.parent; //const parentObj=this.state.graphData[parentName];

      parentObj.children.push(indexNewObj);

      _this._removeLink(currentNode.parent, currentNode);

      newObj.data = indexNewObj; //{"name": newName, "label": "NEW NODE"}

      newObj.depth = currentNode.depth;
      newObj.x = currentNode.x - 100;
      newObj.y = currentNode.y - 100;
      newObj.children = [];
      newObj.children.push(currentNode);
      newObj.parent = currentNode.parent;
      currentNode.parent = newObj;
      currentNode.depth = newObj.depth + 1;
      _this.state.graphData[newName] = newObj; //currentDataProviderNode

      var currentParentDatap = _this.nodeIndex[currentDataProviderNode.parent];
      /*remove the old choldren*/

      var currentParentDatapTmp = currentParentDatap.children.filter(function (dataItem) {
        return dataItem.name !== nodeName;
      });
      currentParentDatap.children = currentParentDatapTmp;
      currentParentDatap.children.push(indexNewObj);
      indexNewObj.children = [];
      indexNewObj.children.push(currentDataProviderNode);
      _this.nodeIndex[currentNode.parent] = currentParentDatapTmp;
      _this.nodeIndex[newName] = indexNewObj;

      if (_this.state.dataEdges[parentObj.data.name + '_' + currentNode.data.name]) {
        delete _this.state.dataEdges[parentObj.data.name + '_' + currentNode.data.name];
      }

      _this.state.dataEdges[newObj.data.name + '_' + currentNode.data.name] = {
        source: currentNode.data.name,
        target: newObj.data.name
      };
      _this.state.dataEdges[parentObj.data.name + '_' + newObj.data.name] = {
        source: parentObj.data.name,
        target: newObj.data.name
      };

      _this.state.nodes.push(newObj);

      break;
  }
};

exports.treeModelApplyAction = treeModelApplyAction;